#!/usr/bin/perl -w

######################################################
# gettable.pl
# 
# (c)2003 Peter Liscovius
# license: GPL
#
# Reads a file with dumps of neighbor- or routingtable of AODV
# for a specified node of Ad hoc simulations of NS-2. 
# (I modified version of AODV-code to dump the tables as xml)
# It sends the latest dump back to the calling program dependent of
# the wanted simtime.
# 
# (de) Die Zahlen am Anfang dienen zur Suche in der Datei und
# und sind monoton steigend. Fr grosse zu 
# durchsuchende Dateien ungnstig.
# 
# The solution is to use a SQL-database or a faster searchalgoritm 
# for indexed files. Use perl standardmodul Search::Dict if possible.
# 
# (de) Wie genau die Tabelle mit der 
# Tabelle zum Zeitpunkt $time in der 
# Simulation wirklich bereinstimmt, 
# hngt davon ab, wie genau nderungen
# an der Nachbartabelle im NS-2 AODV-Code 
# berwacht wurden.
# 
# The precision of the equality of the tables at a given timestamp 
# depends how exact we trace changes of the routing and neighbortables.
# Often only some small value changes(e.g. updating an expire value) but results in 
# a full tabledump to file.
#
# How to call the script wit necessary parameters:  
# http://host/cgipath/gettable.pl?output=typenode=n&time=t&table=table&subdir=path
# 
# Parameters:
#
# output .. "xml"   (text/xml) (default for traceview)
#           "swf" (application/x-shockwave-flash), 
#           "html"  (text/html) or 
#           "text"  (text/plain)
# node   .. integer, the id of a mobile node in ns2 
# time   .. a positive realnumber
# table  .. "n" for neigbortable, "r" for routingtable
# subdir .. path starting from basedir (look in configuration)
#
#
# Example:
# http://localhost/~manet/cgi-bin/ns2/gettable.pl?output=xml&node=0&time=2.345&table=r&subdir=aodvtestdir/out/nodes20-1000x1000
# This calls gettable.pl in cgi-bin-directory of user manet with parameters of the simulation. 
# In traceview this is done by clicking on NT or RT buttons (or automatic by a looping script)

# Structure of ntable-$node.xml und rtable-$node.xml:
#
# time  space xml
#
# 0.355456 <xml..
# 1.243545 <xml..
# 1.345598 <xml..
#
# (deprecated) structure of ntable-$node.htm: 
#
#	0.355456 <table>...</table>
# 1.243545 <table>...</table>
# 1.345598 <table>...</table>
#
#####################################################

use strict;
use CGI;
use CGI::Carp("fatalsToBrowser");

####################################
# start configuration              #
####################################       

# my ns2-simdata on my changable harddisc
# modify
my $basedir="/home/peter/public_html/sims";

# modify path when you installed ming not in default
# system hierarchy
use lib("/aodv/bin/lib/site_perl");

# necessary for swf-output
# for your convenience i put _sans.fdb also 
# in the dir of gettable.pl
my $fontdir="/aodv/fonts";

# only interesting for gettable with HTMLformular in Browser
# can help debugging.
my $ownurl="/~peter/cgi-bin/ns2/gettable.pl";

####################################
# end configuration                #
####################################

# just a hardcoded test with a ready xmlfile
#open (XML,"test.xml");
#print "Content-type: text/xml\n\n";
#print <XML>;
#exit;

# get the vars from client
my $cgi = new CGI;
my $node=$cgi->param("node");
my $time=$cgi->param("time");
my $table=$cgi->param("table");
my $output=$cgi->param("output");
my $subdir=$cgi->param("subdir");

# verify the vars from client
# in this script we don't tolerate false input..
$node  =~ s/\D//g;
$time  =~ s/[^0-9\.]//g;
$table =~ s/[^nr]//g;

# not working find a better one
$output=~ s/[^(xml)|(html)|(text)|(swf)]//ig;

# avoids attacks like ../../../etc/
# fhrenden Pfad aus $subdir entfernen
$subdir=~ s/(\.+)\///g;

if ($node =~ /[^0-9]/)  {&sendError("Wanted node not an integer number.\n");}
if ($node eq "")        {&sendError("Which node? Give me parameter e.g. 'node=1'\n");} 
if ($time =~ /[^0-9\.]/){&sendError("Wanted time not a number.\n");}
if ($time eq "")        {&sendError("At which simulationtime do you want the table? Give me parameter e.g. 'time=1'\n");}
if ($table ne "n" and $table ne "r"){&sendError("Do you want neighbor - or routingtable? Give me parameter 'table=n' or 'table=r' .\n");}
if ($subdir eq "")      {&sendError("No subdirectory given. Give me parameter e.g. 'subdir=nodes10-1000x1000'. \n<br>Basedir on the server is $basedir\n");}



my $tablefile="$basedir/$subdir/".$table."table-$node.xml";
if (not -f "$tablefile" ){ &sendError("File $tablefile not exist/allowed.\n");}


# SLOOOOOOOOW on big files (linear search at moment)
# The files are sorted, so maybe use
#  -perl standardmodul Search::Dict can be used. (best and practical solution if applicable) 
# or
# --binary search with seek, programmed by hand. (good, do you have the code?)
#   How it could work:
#   jump in the middle of file, 
#   choose the correct part dependent of our search criteria ($time) 
#   and jump into the middle of this
# or
# - database solution (e.g. MySQL) 
#  (could be fast, be then everybody who want to use 
#   traceview need to install a database program and must modify other programs
#  (e.g ns-2-code or extra code to put the data into the database)
open (FILE,"<$tablefile") or &sendError("Couldn't open file $tablefile .\n");
my ($simtime,@content);
my $linehit;
while (<FILE>) {
	($simtime,@content) = split(/ /, $_);
	last if ($simtime >= $time);
	$linehit=$_;
}

if    ($output eq "swf"){ &sendSWF; }
elsif ($output eq "html"){ &sendHTML; }
elsif ($output eq "text"){ &sendText; }
else { &sendXML; }


# OLD: This is old stuff, when i wrote htmltables direct into files. 
# Now we use pure xml and must
# convert the data in te xml into a html-page if we want HTML.
# can be done wit XML::DOM 
sub sendHTML{
	print "Content-type: text/html\n\n";
	print "<HTML><BODY BGCOLOR='#FFFFFF'>\n";
	if ($linehit eq ""){print "<P>$table-table of node $node seems to be empty at $time sec.</P>"}
	else { print $linehit;}
	
	# We use it to detect the number of nodes in the sim for the html-formselection. 
	my $tracefile="$basedir/$subdir/tracefile.tr";
	open (TRACE,"<$tracefile") or &sendError("Couldn't open file $tracefile for reading.\n");
	my @row = split(/ /,<TRACE>);
	my $nodes=$row[3];

	my @line= split (/ /,$linehit);

	print "<FORM ACTION='$ownurl'>";
	print "<TABLE BGCOLOR='#EEEEEE' BORDER=0>\n";

	print "<TR><TD ALIGN='right'>Node:</TD><TD><SELECT NAME='node'><OPTION VALUE=$node SELECTED>$node";
	for (my $i=0;$i<$nodes;$i++){
		print "<OPTION VALUE=$i>$i";
	}
	print "</SELECT></TD></TR>\n";

	print "<TR><TD ALIGN='right'>Time:</TD><TD><INPUT TYPE='text' NAME='time' VALUE='$line[0]'></TD></TR>\n";
	print "<TR><TD ALIGN='right'>Subdir:</TD><TD><INPUT TYPE='text' NAME='subdir' VALUE='$subdir'></TD></TR>\n";
	
	# XXX fix me: $table der Anfrage sollte vorausgewhlt sein.
	print "<TR><TD ALIGN='right'>Table:</TD><TD><SELECT NAME='table'>\n";

	my $sel;
	$sel="SELECTED" if ($table eq "n");
	print "<OPTION VALUE='r'>Routingtable<OPTION VALUE='n' $sel>Neighbortable</TD></TR>\n";
	
	print "<TR><TD ALIGN='right'>Output:</TD><TD><SELECT NAME='output'><OPTION VALUE='html' SELECTED>HTML<OPTION VALUE='swf'>SWF(Flash)<OPTION VALUE='text'>Text<OPTION VALUE='xml'>XML</SELECT></TD></TR>";
		
	print "<TR><TD ALIGN='center' COLSPAN=2><INPUT TYPE='Submit' VALUE='Ab damit!'></FORM></TD></TR>\n";
	print "</TABLE></BODY></HTML>";
 	exit;
}

# just take the line and send a nice xmlfile to flashplayer
sub sendXML{
	# bis zum ersten Leerzeichen alles wegschneiden 
	# (Zeitstempel fr Suche in Datei gehrt nicht zum XML)
	$linehit=~ s/(\S+)(\s+)//;
	
	#print "Content-type: text/html\n\n";
	# We just send out the xml to the swf6 player/plugin
	print "Content-type: text/xml\n\n";
	print '<?xml version="1.0" encoding="iso-8859-1" ?>';
	print $linehit;

}

# When we want render tables of data into flashmovie
sub sendSWF{
	
	print "Content-type: application/x-shockwave-flash\n\n";
	#print STDERR "in sendSWF: time=$time subdir=$tracefile";
	
	# these 2 things will we need later when building the tables in ming.
	# currently not implemented so uncommented
	#use XML::DOM;
	#require("/aodv/bin/traceviewgui.pl");
	
	use SWF qw(:ALL);
	SWF::setVersion(4);
		
	my $font = new SWF::Font("$fontdir/_sans.fdb");

	my $m=new SWF::Movie;
	$m->setDimension(400,200);
	$m->setBackground(0x66,0xff,0x99);
	
	my $window = new SWF::MovieClip();
	my ($lastupdate,@line) = split (/ /,$linehit);
	$linehit=~ s/(\S+)(\s+)//;
	
	# $linehit now contains only our xml-file
	# we can now analyze it wit XML::DOM and can build a nice 
	# table with ming. 
	
	my $time= new SWF::Text();
	$time->setFont($font);
	$time->setHeight(12);
	$time->setColor(0x33,0,0);
	$time->addString("Last update of ".$table."table: ".$lastupdate." sec.");
	my $time_i = $window->add($time);
	$time_i->move(10,20);
	$window->nextFrame;
	my $window_i = $m->add($window);
	$m->nextFrame;
	$m->output;
	#$m->save($tablefile.".swf");
}


sub sendText{
	print "Content-type: text/plain\n\n";
	print "This is just a dump of the xml that is normally sent to the traceview frontend:\n\n";
	print '<?xml version="1.0" encoding="iso-8859-1" ?>';
	print $linehit;
	
}

sub sendError{
	my $error=shift;
	if ($output eq "swf"){ &sendSWFError("$error");}
	elsif ($output eq "xml"){ &sendXMLError("$error");}
	else {&sendHTMLError("$error");}
	exit;
}

sub sendHTMLError{
	my $error=shift;
	print "Content-type: text/html\n\n";
	print "<H1>$error</H1>";
	# debug
	print "time=$time<br>\noutput=$output<br>\nnode=$node<br>\ntable=$table<br>\nsubdir=$subdir";
}

sub sendXMLError{
	my $error=shift;
	print "Content-type: text/xml\n\n";
	print '<?xml version="1.0" encoding="iso-8859-1" ?>'."<error>$error</error>\n";

}

sub sendSWFError{
	# only send a minimal redbackgrounded swf.
	print "Content-type: application/x-shockwave-flash\n\n";
	use SWF qw(:ALL);
	my $m=new SWF::Movie;
	$m->setDimension(200,100);
	$m->setBackground(0xff,0,0);
	$m->nextFrame;
	$m->output;
}
