################################################
# Author: Peter Liscovius                      #    
# (c) 2003                                     #
#                                              #
# software is under GPL licence                # 
#                                              #
# functions for analyzing ns-2 new-trace files #
# currently focussed on AODV                   #
################################################

# new objectoriented interface:
# use TraceStat;
# my $stat=new TraceStat;
# $stat->makeStat("datei");
# $stat->saveAsText("datei.txt");
# $stat->saveAsXML("datei.xml");
# print $stat->{AGT_CBR}{s};
#
# look into makestat.pl for an example.
# $maxspeed is optional too. Just to have that value in the statfile.

use strict;

package TraceStat;

my $VERSION="0.4.1";

sub new {
my $class = shift;
my $self= {};

$self->{AGT_CBR} = {s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{AGT_TCP} = {s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{AGT_UNKNOWN}  = {s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};

$self->{RTR_AODVRREQ} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{RTR_AODVRREP} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{RTR_AODVHELLO}={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{RTR_AODVERROR}={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};							 
$self->{RTR_AODVUNKNOWN}=0;

$self->{RTR_CBR}      ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{RTR_TCP}      ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{RTR_UNKNOWN}  ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};

$self->{MAC_AODVRREQ} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{MAC_AODVRREP} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{MAC_AODVHELLO}={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{MAC_AODVERROR}={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};							 
$self->{MAC_AODVUNKNOWN}=0;
$self->{MAC_CBR}     ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{MAC_TCP}     ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{MAC_UNKNOWN} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};							 

# for drops caused by IFQ
$self->{IFQ} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};

$self->{UNKNOWN} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};

# different tracelevels RTR, AGT, MAC
$self->{sumAGT} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{sumRTR} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
$self->{sumMAC} ={s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};

$self->{nodes}=0;
$self->{errors}=0;

$self->{packet_id}=0;
$self->{highest_packet_id}=0;

$self->{AGT_start_time}=[""]; 
$self->{AGT_end_time}=[""];

$self->{sumAGT_end_to_end_delay}=0;
$self->{AGT_end_to_end_delay}=0;
$self->{maxspeed}=0;

bless ($self,	$class);
return $self;
}

=head2

interface: $stat->makeStat(tracefile,statisticfile); 

Calculates stat for MAC, AODV and CBR. TCP too, but maybe wrong.

Not very clever written. Please make it better.

=cut

sub makeStat {
	my $self =shift;
	#$maxspeed is optional
	my ($file,$statfile,$maxspeed)=@_;
	
	print "Opening $file and generating statistic ..\n";
	
	# currently unixspecific command. maybe there exist a better way(e.g a Perlmodule?)
	my $mem=`free -t | grep Total:`;
	print $mem."\n";
	my @mem= split(/\s+/,$mem);
	print "Available memory: ".($mem[3]/1024)." MB\n";
	
	my $filesize= -s $file;
	print "  Tracefile size: ".($filesize/1024/1024)." MB\n"; 
	my $lines=`wc -l $file`;
	$lines =~ s/^\s+//;
	my @lines=split( /\s+/,$lines);
	
	print "$file has $lines[0] lines.\n";
	
	if ($filesize > $mem[3]) {
		#
	}
	else {
		#open (FILE, "<$file") || die("Couldn't open file $file in TraceStat::makeStat()..\n");
		# Problem when big files and small RAM in your workstation. 
		# Then better read the file linebyline while doing stat but script then slower.
		# hier eval{} benutzen fuer Ausnahmebehandlung oder vorher RAM des Rechners und 
		# Dateigrsse testen?
		#my @file = <FILE>; 
	
		#my $mem=`free -t | grep "Total:"`;
		#my @mem= split(/\s/,$mem);
		#print "Available memory: ".$mem[3]."\n";	
		#close FILE;
	}
	
	open (FILE, "<$file") || die("Couldn't open file $file: $!\n");
	#my $maxspeed;
	#my @firstline=split(/ /,$file[0]);
	my @firstline=split(/ /,<FILE>);
	$self->{nodes}=$firstline[3];

my $time;
my $readlines=0;
#foreach (@file)	{
while (<FILE>)	{
	
	my $rest= $readlines%10000;
	if ($rest == 0) {
		printf( "%d (%.2f %%)\n", $readlines, ($readlines/$lines[0]*100));
	}
	$readlines++;
	# detect the tracelevel
	if ($_ =~ /^[srfd]\s/){
		($time,$self->{packet_id})=getBaseInfos($_);
	
	} else {
		print(STDERR "Irregular line in tracefile:\n $_");
		next;
	}
	#print STDERR "$time $packed_id\n";
	if ($self->{packet_id} > $self->{highest_packet_id}) {$self->{highest_packet_id} = $self->{packet_id};}
	
	if ($_ =~ /-Nl AGT/){ 
		# We are tracing in the Agent-layer
		# Counting the  packets here doesn't automatically means it is send over the air. 
		# Could be also dropped in interfacequeue e.g. 
		if ($_ =~ /-It cbr /i){	
			#print STDERR "Constant Bit Rate (UDP?)-packets";
			if ($_ =~ /^s/){	
				$self->{AGT_CBR}{sB}+=getPacketSize($_);
				$self->{AGT_CBR}{s}++;
				if ( not defined $self->{AGT_start_time}[$self->{packet_id}]) { $self->{AGT_start_time}[$self->{packet_id}] = $time;}
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{AGT_CBR}{rB}+=getPacketSize($_);
				$self->{AGT_CBR}{r}++;
				if (not defined $self->{AGT_end_time}[$self->{packet_id}]) {$self->{AGT_end_time}[$self->{packet_id}] = $time; }
			} 
			elsif ($_ =~ /^f/){  
				$self->{AGT_CBR}{fB}+=getPacketSize($_);
				$self->{AGT_CBR}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				$self->{AGT_CBR}{dB}+=getPacketSize($_);
				$self->{AGT_CBR}{d}++;
			}
			else {	
				$self->{AGT_CBR}{uB}+=getPacketSize($_);
				$self->{AGT_CBR}{u}++;
			}	
		} 
		elsif ($_ =~ /-It tcp /i) { 
			if ($_ =~ /^s/){	
				$self->{AGT_TCP}{sB}+=getPacketSize($_);
				$self->{AGT_TCP}{s}++;
				if ( not defined $self->{AGT_start_time}[$self->{packet_id}]) {$self->{AGT_start_time}[$self->{packet_id}] = $time;}
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{AGT_TCP}{rB}+=getPacketSize($_);
				$self->{AGT_TCP}{r}++;
				if (not defined $self->{AGT_end_time}[$self->{packet_id}]) {$self->{AGT_end_time}[$self->{packet_id}] = $time; }
			} 
			elsif ($_ =~ /^f/){  
				$self->{AGT_TCP}{fB}+=getPacketSize($_);
				$self->{AGT_TCP}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				$self->{AGT_TCP}{dB}+=getPacketSize($_);
				$self->{AGT_TCP}{d}++;
			}
			else {	
				$self->{AGT_TCP}{uB}+=getPacketSize($_);
				$self->{AGT_TCP}{u}++;
			}	
		}
		else {
			if ($_ =~ /^s\s/)    {$self->{AGT_UNKNOWN}{s}++; $self->{AGT_UNKNOWN}{sB}+=getPacketSize($_);}
			elsif ($_ =~ /^r\s/) {$self->{AGT_UNKNOWN}{r}++; $self->{AGT_UNKNOWN}{rB}+=getPacketSize($_);}
			elsif ($_ =~ /^f\s/) {$self->{AGT_UNKNOWN}{f}++; $self->{AGT_UNKNOWN}{fB}+=getPacketSize($_);}
			elsif ($_ =~ /^d\s/) {$self->{AGT_UNKNOWN}{d}++; $self->{AGT_UNKNOWN}{dB}+=getPacketSize($_);}
			else {$self->{AGT_UNKNOWN}{u}++;}
		}
	}
	elsif($_ =~ /-Nl RTR/){
	  # We are tracing in the Router-layer
		if ($_ =~ /-It cbr /i){	
			#print STDERR "Constant Bit Rate (UDP?)-packets";
			if ($_ =~ /^s/){	
				$self->{RTR_CBR}{sB}+=getPacketSize($_);
				$self->{RTR_CBR}{s}++;
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{RTR_CBR}{rB}+=getPacketSize($_);
				$self->{RTR_CBR}{r}++;
			} 
			elsif ($_ =~ /^f/){  
				$self->{RTR_CBR}{fB}+=getPacketSize($_);
				$self->{RTR_CBR}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				$self->{RTR_CBR}{dB}+=getPacketSize($_);
				$self->{RTR_CBR}{d}++;
			}
			else {	
				$self->{RTR_CBR}{uB}+=getPacketSize($_);
				$self->{RTR_CBR}{u}++;
			}	
		} 
		elsif ($_ =~ /-It tcp /i) { 
			if ($_ =~ /^s/){	
				$self->{RTR_TCP}{sB}+=getPacketSize($_);
				$self->{RTR_TCP}{s}++;
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{RTR_TCP}{rB}+=getPacketSize($_);
				$self->{RTR_TCP}{r}++;
			} 
			elsif ($_ =~ /^f/){  
				$self->{RTR_TCP}{fB}+=getPacketSize($_);
				$self->{RTR_TCP}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				$self->{RTR_TCP}{dB}+=getPacketSize($_);
				$self->{RTR_TCP}{d}++;
			}
			else {	
				$self->{RTR_TCP}{uB}+=getPacketSize($_);
				$self->{RTR_TCP}{u}++;
			}	
		}
		elsif ($_ =~ /-It AODV/i) {
			#ok AODV
			if ($_ =~ /-Pc REQUEST/){ 	
				#print STDERR "AODV-REQUEST\n $_"; 
				if ($_ =~ /^s/){	
					#print STDERR " was sent\n";
					$self->{RTR_AODVRREQ}{sB}+=getPacketSize($_);
					$self->{RTR_AODVRREQ}{s}++;
				} 
				elsif ($_ =~ /^r/){	
					#print STDERR " was received\n";
					$self->{RTR_AODVRREQ}{rB}+=getPacketSize($_);
					$self->{RTR_AODVRREQ}{r}++;
				}
				elsif ($_ =~ /^f/){  	
					#maybe forward or unknown type
					#print STDERR " was forwarded.\n";
					$self->{RTR_AODVRREQ}{fB}+=getPacketSize($_);
					$self->{RTR_AODVRREQ}{f}++;
				}
				elsif ($_ =~ /^d/){	
					#print STDERR " was dropped.\n";
					$self->{RTR_AODVRREQ}{dB}+=getPacketSize($_);
					$self->{RTR_AODVRREQ}{d}++;
				}
				else {  	
					#unknown action
					#print STDERR " what?.\n";
					$self->{RTR_AODVRREQ}{uB}+=getPacketSize($_);
					$self->{RTR_AODVRREQ}{u}++; 
				}
			} 
			elsif ($_ =~ /-Pc REPLY/){ 	
				#print STDERR "AODV-REPLY"; 
				if ($_ =~ /^s/)
				{	#print STDERR " was sent.\n";
					$self->{RTR_AODVRREP}{sB}+=getPacketSize($_);
					$self->{RTR_AODVRREP}{s}++;
				}
				elsif ($_ =~ /^r/)
				{	#print STDERR " was received.\n";
					$self->{RTR_AODVRREP}{rB}+=getPacketSize($_);
					$self->{RTR_AODVRREP}{r}++;
				} 
				elsif ($_ =~ /^f/)
				{  	
					#print STDERR " was forwarded.\n";
					$self->{RTR_AODVRREP}{fB}+=getPacketSize($_);
					$self->{RTR_AODVRREP}{f}++;
				}
				elsif ($_ =~ /^d/)
				{  	
					#print STDERR " was dropped.\n";
					$self->{RTR_AODVRREP}{dB}+=getPacketSize($_);
					$self->{RTR_AODVRREP}{d}++;
				}
				else 
				{ 	#print STDERR " what?.\n";
					$self->{RTR_AODVRREP}{uB}+=getPacketSize($_);
					$self->{RTR_AODVRREP}{u}++;
				}	
			} 
			elsif ($_ =~ /-Pc ERROR/){ 	
				#print STDERR "AODV-ERROR"; 
				if ($_ =~ /^s/)
				{	#print STDERR " was sent\n";
					$self->{RTR_AODVERROR}{sB}+=getPacketSize($_);
					$self->{RTR_AODVERROR}{s}++;
				}
				elsif ($_ =~ /^r/)
				{	#print STDERR " was received\n";
					$self->{RTR_AODVERROR}{rB}+=getPacketSize($_);
					$self->{RTR_AODVERROR}{r}++;
				} 
				elsif ($_ =~ /^f/)
				{  	
					#print STDERR " was forwarded.\n";
					$self->{RTR_AODVERROR}{fB}+=getPacketSize($_);
					$self->{RTR_AODVERROR}{f}++;
				}
				elsif ($_ =~ /^d/)
				{  	
					#print STDERR " was dropped.\n";
					$self->{RTR_AODVERROR}{dB}+=getPacketSize($_);
					$self->{RTR_AODVERROR}{d}++;
				}
				else
				{	#print STDERR " what?.\n";
					$self->{RTR_AODVERROR}{uB}+=getPacketSize($_);
					$self->{RTR_AODVERROR}{u}++;
				}
	
			} 
			elsif ($_ =~ /-Pc HELLO/){ 	
				#print STDERR "AODV-HELLO"; 
				if ($_ =~ /^s/){	
					#print STDERR " was sent\n";
					$self->{RTR_AODVHELLO}{sB}+=getPacketSize($_);
					$self->{RTR_AODVHELLO}{s}++;
				}
				elsif ($_ =~ /^r/){	
					#print STDERR " was received\n";			
					$self->{RTR_AODVHELLO}{rB}+=getPacketSize($_);
					$self->{RTR_AODVHELLO}{r}++;
				} 
				elsif ($_ =~ /^f/){  	
					#print STDERR " was forwarded.\n";			
					$self->{RTR_AODVHELLO}{fB}+=getPacketSize($_);
					$self->{RTR_AODVHELLO}{f}++;
				}
				elsif ($_ =~ /^d/){ 	
					#print STDERR " was dropped.\n";
					$self->{RTR_AODVHELLO}{dB}+=getPacketSize($_);
					$self->{RTR_AODVHELLO}{d}++;
				}
				else{ 	
					#print STDERR " what?\n";
					$self->{RTR_AODVHELLO}{uB}+=getPacketSize($_);
					$self->{RTR_AODVHELLO}{u}++;
				}	
			}
			else {
				$self->{RTR_AODVUNKNOWN}++;
			}
		}		
		else {
			if ($_ =~ /^s\s/)    {$self->{RTR_UNKNOWN}{s}++; $self->{RTR_UNKNOWN}{sB}+=getPacketSize($_);}
			elsif ($_ =~ /^r\s/) {$self->{RTR_UNKNOWN}{r}++; $self->{RTR_UNKNOWN}{rB}+=getPacketSize($_);}
			elsif ($_ =~ /^f\s/) {$self->{RTR_UNKNOWN}{f}++; $self->{RTR_UNKNOWN}{fB}+=getPacketSize($_);}
			elsif ($_ =~ /^d\s/) {$self->{RTR_UNKNOWN}{d}++; $self->{RTR_UNKNOWN}{dB}+=getPacketSize($_);}
			else {$self->{AGT_UNKNOWN}{u}++;}
			#print "$_\n";
		}
	}
	elsif($_ =~ /-Nl MAC/){
		#we are tracing in the MediumACcess-layer
		if ($_ =~ /-It cbr /i){	
			#print STDERR "Constant Bit Rate (UDP?)-packets";
			if ($_ =~ /^s/){	
				#print STDERR " was sent.\n";
				$self->{MAC_CBR}{sB}+=getPacketSize($_);
				$self->{MAC_CBR}{s}++;
				if (not defined $self->{MAC_start_time}[$self->{packet_id}]) { $self->{MAC_start_time}[$self->{packet_id}] = $time; }
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{MAC_CBR}{rB}+=getPacketSize($_);
				$self->{MAC_CBR}{r}++;
				#if $end_time[$packet_id] {};
				if (not defined $self->{MAC_end_time}[$self->{packet_id}]) { $self->{MAC_end_time}[$self->{packet_id}] = $time;}
			} 
			elsif ($_ =~ /^f/){  
				#print STDERR " was forwarded.\n";
				$self->{MAC_CBR}{fB}+=getPacketSize($_);
				$self->{MAC_CBR}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				#print STDERR " was dropped.\n";
				$self->{MAC_CBR}{dB}+=getPacketSize($_);
				$self->{MAC_CBR}{d}++;
			}
			else {	
				#print STDERR " unknown action!\n $_\n";
				$self->{MAC_CBR}{uB}+=getPacketSize($_);
				$self->{MAC_CBR}{u}++;
			}	
		} 
		elsif ($_ =~ /-It tcp /i) { 
			#print STDERR "Constant Bit Rate (UDP?)-packets";
			if ($_ =~ /^s/){	
				#print STDERR " was sent.\n";
				$self->{MAC_TCP}{sB}+=getPacketSize($_);
				$self->{MAC_TCP}{s}++;
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{MAC_TCP}{rB}+=getPacketSize($_);
				$self->{MAC_TCP}{r}++;
			} 
			elsif ($_ =~ /^f/){  
				#print STDERR " was forwarded.\n";
				$self->{MAC_TCP}{fB}+=getPacketSize($_);
				$self->{MAC_TCP}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				#print STDERR " was dropped.\n";
				$self->{MAC_TCP}{dB}+=getPacketSize($_);
				$self->{MAC_TCP}{d}++;
			}
			else {	
				#print STDERR " unknown action!\n $_\n";
				$self->{MAC_TCP}{uB}+=getPacketSize($_);
				$self->{MAC_TCP}{u}++;
			}	
		}
		elsif ($_ =~ /-It AODV/i) {
			#ok AODV
			if ($_ =~ /-Pc REQUEST/){ 	
				#print STDERR "AODV-REQUEST\n $_"; 
				if ($_ =~ /^s/){	
					#print STDERR " was sent\n";
					$self->{MAC_AODVRREQ}{sB}+=getPacketSize($_);
					$self->{MAC_AODVRREQ}{s}++;
				} 
				elsif ($_ =~ /^r/){	
					#print STDERR " was received\n";
					$self->{MAC_AODVRREQ}{rB}+=getPacketSize($_);
					$self->{MAC_AODVRREQ}{r}++;
				}
				elsif ($_ =~ /^f/){  	
					#print STDERR " was forwarded.\n";
					$self->{MAC_AODVRREQ}{fB}+=getPacketSize($_);
					$self->{MAC_AODVRREQ}{f}++;
				}
				elsif ($_ =~ /^d/){	
					#print STDERR " was dropped.\n";
					$self->{MAC_AODVRREQ}{dB}+=getPacketSize($_);
					$self->{MAC_AODVRREQ}{d}++;
				}
				else {  	
					#unknown action
					#print STDERR " what?.\n";
					$self->{MAC_AODVRREQ}{uB}+=getPacketSize($_);
					$self->{MAC_AODVRREQ}{u}++; 
				}
			} 
			elsif ($_ =~ /-Pc REPLY/){ 	
				#print STDERR "AODV-REPLY"; 
				if ($_ =~ /^s/)
				{	#print STDERR " was sent.\n";
					$self->{MAC_AODVRREP}{sB}+=getPacketSize($_);
					$self->{MAC_AODVRREP}{s}++;
				}
				elsif ($_ =~ /^r/)
				{	#print STDERR " was received.\n";
					$self->{MAC_AODVRREP}{rB}+=getPacketSize($_);
					$self->{MAC_AODVRREP}{r}++;
				} 
				elsif ($_ =~ /^f/)
				{  	#maybe forward or unknown type
					#print STDERR " was forwarded.\n";
					$self->{MAC_AODVRREP}{fB}+=getPacketSize($_);
					$self->{MAC_AODVRREP}{f}++;
				}
				elsif ($_ =~ /^d/)
				{  	#maybe forward or unknown type
					#print STDERR " was dropped.\n";
					$self->{MAC_AODVRREP}{dB}+=getPacketSize($_);
					$self->{MAC_AODVRREP}{d}++;
				}
				else 
				{ 	#print STDERR " what?.\n";
					$self->{MAC_AODVRREP}{uB}+=getPacketSize($_);
					$self->{MAC_AODVRREP}{u}++;
				}	
			} 
			elsif ($_ =~ /-Pc ERROR/){ 	
				#print STDERR "AODV-ERROR"; 
				if ($_ =~ /^s/)
				{	#print STDERR " was sent\n";
					$self->{MAC_AODVERROR}{sB}+=getPacketSize($_);
					$self->{MAC_AODVERROR}{s}++;
				}
				elsif ($_ =~ /^r/)
				{	#print STDERR " was received\n";
					$self->{MAC_AODVERROR}{rB}+=getPacketSize($_);
					$self->{MAC_AODVERROR}{r}++;
				} 
				elsif ($_ =~ /^f/)
				{  	#maybe forward or unknown type
					#print STDERR " was forwarded.\n";
					$self->{MAC_AODVERROR}{fB}+=getPacketSize($_);
					$self->{MAC_AODVERROR}{f}++;
				}
				elsif ($_ =~ /^d/)
				{  	#maybe forward or unknown type
					#print STDERR " was dropped.\n";
					$self->{MAC_AODVERROR}{dB}+=getPacketSize($_);
					$self->{MAC_AODVERROR}{d}++;
				}
				else
				{	#print STDERR " what?.\n";
					$self->{MAC_AODVERROR}{uB}+=getPacketSize($_);
					$self->{MAC_AODVERROR}{u}++;
				}
	
			} 
			elsif ($_ =~ /-Pc HELLO/){ 	
				#print STDERR "AODV-HELLO"; 
				if ($_ =~ /^s/){	
					#print STDERR " was sent\n";
					$self->{MAC_AODVHELLO}{sB}+=getPacketSize($_);
					$self->{MAC_AODVHELLO}{s}++;
				}
				elsif ($_ =~ /^r/){	
					#print STDERR " was received\n";			
					$self->{MAC_AODVHELLO}{rB}+=getPacketSize($_);
					$self->{MAC_AODVHELLO}{r}++;
				} 
				elsif ($_ =~ /^f/){  	
					#print STDERR " was forwarded.\n";			
					$self->{MAC_AODVHELLO}{fB}+=getPacketSize($_);
					$self->{MAC_AODVHELLO}{f}++;
				}
				elsif ($_ =~ /^d/){ 	
					#print STDERR " was dropped.\n";
					$self->{MAC_AODVHELLO}{dB}+=getPacketSize($_);
					$self->{MAC_AODVHELLO}{d}++;
				}
				else{ 	
					#print STDERR " what?\n";
					$self->{MAC_AODVHELLO}{u}++;
				}	
			}	
			else{
				#unknown AODV packet.Maybe a new ns2-implemention of Reply-Ack?
				#add if so.
				$self->{MAC_AODVUNKNOWN}++;
			}
		}
		else 
		{	
			#just log sent receive forward drop events
			if ($_ =~ /^s/) { $self->{MAC_UNKNOWN}{s}++; }
			elsif ($_ =~ /^r/) { $self->{MAC_UNKNOWN}{r}++;} 
			elsif ($_ =~ /^f/) { $self->{MAC_UNKNOWN}{f}++;}
			elsif ($_ =~ /^d/) { $self->{MAC_UNKNOWN}{d}++; }
			else { $self->{MAC_UNKNOWN}{u}++;} 
			#print "MAC $_\n";	
		}
	}
	elsif($_ =~ /-Nl IFQ/){
			#just log sent receive forward drop events
			if ($_ =~ /^s/) { $self->{IFQ}{s}++; }
			elsif ($_ =~ /^r/) { $self->{IFQ}{r}++;} 
			elsif ($_ =~ /^f/) { $self->{IFQ}{f}++;}
			elsif ($_ =~ /^d/) { $self->{IFQ}{d}++; }
			else { $self->{IFQ}{u}++;} 
			#print "IFQ $_\n";
	
	}
	else{
		if ($_ =~ /^s/) { $self->{UNKNOWN}{s}++; }
		elsif ($_ =~ /^r/) { $self->{UNKNOWN}{r}++;} 
		elsif ($_ =~ /^f/) { $self->{UNKNOWN}{f}++;}
		elsif ($_ =~ /^d/) { $self->{UNKNOWN}{d}++; }
		else { $self->{UNKNOWN}{d}++;}
		print "$_\n";
	}
}
close(FILE);

	$self->{RTR_AODV}{s} = $self->{RTR_AODVRREQ}{s} + $self->{RTR_AODVRREP}{s} + $self->{RTR_AODVHELLO}{s} + $self->{RTR_AODVERROR}{s};
	$self->{RTR_AODV}{r} = $self->{RTR_AODVRREQ}{r} + $self->{RTR_AODVRREP}{r} + $self->{RTR_AODVHELLO}{r} + $self->{RTR_AODVERROR}{r};
	$self->{RTR_AODV}{f} = $self->{RTR_AODVRREQ}{f} + $self->{RTR_AODVRREP}{f} + $self->{RTR_AODVHELLO}{f} + $self->{RTR_AODVERROR}{f};
	$self->{RTR_AODV}{d} = $self->{RTR_AODVRREQ}{d} + $self->{RTR_AODVRREP}{d} + $self->{RTR_AODVHELLO}{d} + $self->{RTR_AODVERROR}{d};
	$self->{RTR_AODV}{u} = $self->{RTR_AODVRREQ}{u} + $self->{RTR_AODVRREP}{u} + $self->{RTR_AODVHELLO}{u} + $self->{RTR_AODVERROR}{u};
	
	$self->{RTR_AODV}{sB} = $self->{RTR_AODVRREQ}{sB} + $self->{RTR_AODVRREP}{sB} + $self->{RTR_AODVHELLO}{sB} + $self->{RTR_AODVERROR}{sB};
	$self->{RTR_AODV}{rB} = $self->{RTR_AODVRREQ}{rB} + $self->{RTR_AODVRREP}{rB} + $self->{RTR_AODVHELLO}{rB} + $self->{RTR_AODVERROR}{rB};
	$self->{RTR_AODV}{fB} = $self->{RTR_AODVRREQ}{fB} + $self->{RTR_AODVRREP}{fB} + $self->{RTR_AODVHELLO}{fB} + $self->{RTR_AODVERROR}{fB};
	$self->{RTR_AODV}{dB} = $self->{RTR_AODVRREQ}{dB} + $self->{RTR_AODVRREP}{dB} + $self->{RTR_AODVHELLO}{dB} + $self->{RTR_AODVERROR}{dB};
	$self->{RTR_AODV}{uB} = $self->{RTR_AODVRREQ}{uB} + $self->{RTR_AODVRREP}{uB} + $self->{RTR_AODVHELLO}{uB} + $self->{RTR_AODVERROR}{uB};

	$self->{MAC_AODV}{s} = $self->{MAC_AODVRREQ}{s} + $self->{MAC_AODVRREP}{s} + $self->{MAC_AODVHELLO}{s} + $self->{MAC_AODVERROR}{s};
	$self->{MAC_AODV}{r} = $self->{MAC_AODVRREQ}{r} + $self->{MAC_AODVRREP}{r} + $self->{MAC_AODVHELLO}{r} + $self->{MAC_AODVERROR}{r};
	$self->{MAC_AODV}{f} = $self->{MAC_AODVRREQ}{f} + $self->{MAC_AODVRREP}{f} + $self->{MAC_AODVHELLO}{f} + $self->{MAC_AODVERROR}{f};
	$self->{MAC_AODV}{d} = $self->{MAC_AODVRREQ}{d} + $self->{MAC_AODVRREP}{d} + $self->{MAC_AODVHELLO}{d} + $self->{MAC_AODVERROR}{d};
	$self->{MAC_AODV}{u} = $self->{MAC_AODVRREQ}{u} + $self->{MAC_AODVRREP}{u} + $self->{MAC_AODVHELLO}{u} + $self->{MAC_AODVERROR}{u};
	
	$self->{MAC_AODV}{sB} = $self->{MAC_AODVRREQ}{sB} + $self->{MAC_AODVRREP}{sB} + $self->{MAC_AODVHELLO}{sB} + $self->{MAC_AODVERROR}{sB};
	$self->{MAC_AODV}{rB} = $self->{MAC_AODVRREQ}{rB} + $self->{MAC_AODVRREP}{rB} + $self->{MAC_AODVHELLO}{rB} + $self->{MAC_AODVERROR}{rB};
	$self->{MAC_AODV}{fB} = $self->{MAC_AODVRREQ}{fB} + $self->{MAC_AODVRREP}{fB} + $self->{MAC_AODVHELLO}{fB} + $self->{MAC_AODVERROR}{fB};
	$self->{MAC_AODV}{dB} = $self->{MAC_AODVRREQ}{dB} + $self->{MAC_AODVRREP}{dB} + $self->{MAC_AODVHELLO}{dB} + $self->{MAC_AODVERROR}{dB};
	$self->{MAC_AODV}{uB} = $self->{MAC_AODVRREQ}{uB} + $self->{MAC_AODVRREP}{uB} + $self->{MAC_AODVHELLO}{uB} + $self->{MAC_AODVERROR}{uB};

	# sum AGT level
	$self->{sumAGT}{s} = $self->{AGT_CBR}{s} + $self->{AGT_TCP}{s};
	$self->{sumAGT}{r} = $self->{AGT_CBR}{r} + $self->{AGT_TCP}{r};
	$self->{sumAGT}{f} = $self->{AGT_CBR}{f} + $self->{AGT_TCP}{f};
	$self->{sumAGT}{d} = $self->{AGT_CBR}{d} + $self->{AGT_TCP}{d};	
	$self->{sumAGT}{u} = $self->{AGT_CBR}{u} + $self->{AGT_TCP}{u};	
	
	$self->{sumAGT}{sB} = $self->{AGT_CBR}{sB} + $self->{AGT_TCP}{rB};
	$self->{sumAGT}{rB} = $self->{AGT_CBR}{rB} + $self->{AGT_TCP}{rB};
	$self->{sumAGT}{fB} = $self->{AGT_CBR}{fB} + $self->{AGT_TCP}{fB};
	$self->{sumAGT}{dB} = $self->{AGT_CBR}{dB} + $self->{AGT_TCP}{dB};	
	$self->{sumAGT}{uB} = $self->{AGT_CBR}{uB} + $self->{AGT_TCP}{uB};
	
	# sum RTR level
	$self->{sumRTR}{s} = $self->{RTR_AODV}{s} + $self->{RTR_CBR}{s} + $self->{RTR_TCP}{s}; 
	$self->{sumRTR}{r} = $self->{RTR_AODV}{r} + $self->{RTR_CBR}{r} + $self->{RTR_TCP}{r};
	$self->{sumRTR}{f} = $self->{RTR_AODV}{f} + $self->{RTR_CBR}{f} + $self->{RTR_TCP}{f};
	$self->{sumRTR}{d} = $self->{RTR_AODV}{d} + $self->{RTR_CBR}{d} + $self->{RTR_TCP}{d};
	$self->{sumRTR}{u} = $self->{RTR_AODV}{u} + $self->{RTR_CBR}{u} + $self->{RTR_TCP}{u};
	
	$self->{sumRTR}{rB} = $self->{RTR_AODV}{rB} ; 
	$self->{sumRTR}{rB} = $self->{RTR_AODV}{rB} ;
	$self->{sumRTR}{fB} = $self->{RTR_AODV}{fB} ;
	$self->{sumRTR}{dB} = $self->{RTR_AODV}{dB} ;
	$self->{sumRTR}{uB} = $self->{RTR_AODV}{uB} ;
	
	# sum MAC level
	$self->{sumMAC}{s} = $self->{MAC_AODV}{s} + $self->{MAC_CBR}{s} + $self->{MAC_TCP}{s} ; 
	$self->{sumMAC}{r} = $self->{MAC_AODV}{r} + $self->{MAC_CBR}{r} + $self->{MAC_TCP}{r} ;
	$self->{sumMAC}{f} = $self->{MAC_AODV}{f} + $self->{MAC_CBR}{f} + $self->{MAC_TCP}{f} ;
	$self->{sumMAC}{d} = $self->{MAC_AODV}{d} + $self->{MAC_CBR}{d} + $self->{MAC_TCP}{d} ;
	$self->{sumMAC}{u} = $self->{MAC_AODV}{u} + $self->{MAC_CBR}{u} + $self->{MAC_TCP}{u} ;
	
	$self->{sumMAC}{sB} = $self->{MAC_AODV}{sB} + $self->{MAC_CBR}{sB} + $self->{MAC_TCP}{sB} ; 
	$self->{sumMAC}{rB} = $self->{MAC_AODV}{rB} + $self->{MAC_CBR}{rB} + $self->{MAC_TCP}{rB} ;
	$self->{sumMAC}{fB} = $self->{MAC_AODV}{fB} + $self->{MAC_CBR}{fB} + $self->{MAC_TCP}{fB} ;
	$self->{sumMAC}{dB} = $self->{MAC_AODV}{dB} + $self->{MAC_CBR}{dB} + $self->{MAC_TCP}{dB} ;
	$self->{sumMAC}{uB} = $self->{MAC_AODV}{uB} + $self->{MAC_CBR}{uB} + $self->{MAC_TCP}{uB} ;

print(STDERR "calculating delay ..\n");
$self->{sumAGT_end_to_end_delay}=0;
for (my $packet_id = 0; $packet_id <= $self->{highest_packet_id} ; $packet_id++) {
	if ((defined $self->{AGT_end_time}[$packet_id]) && (defined $self->{AGT_start_time}[$packet_id])) {
		my $packet_duration = $self->{AGT_end_time}[$packet_id] - $self->{AGT_start_time}[$packet_id];
		print(STDERR "pkt: $packet_id delay: $packet_duration\n");
		if ($packet_duration >0) {$self->{sumAGT_end_to_end_delay} += $packet_duration;}
	}
}
if ( ($self->{AGT_CBR}{r}+$self->{AGT_TCP}{r}) <1) { $self->{AGT_end_to_end_delay}=0;}
else { $self->{AGT_end_to_end_delay} = $self->{sumAGT_end_to_end_delay} / ($self->{AGT_CBR}{r}+$self->{AGT_TCP}{r}); }

}

=head2 saveAsXML

 UNFINISHED!

 Please take a review and make a better xml-tree. NS-2 Wireless Traceview does not interprete this xmlfile.
 Its just for the future or other projects.

=cut

sub saveAsXML {
	my $self =shift;
	my $statfile=shift;

	#if ($statfile eq ""){$statfile="tracefile.xml";}
	if ($statfile eq "") { die("No parameter given where the statfile should be saved.\n"); }
	
	print(STDERR "save statistics at $statfile .\n");
	open (STATIST,">$statfile") or die("\n Couldn't open $statfile .\n");

	print STATIST '<?xml version="1.0"?>'."\n<stat version='$VERSION'>\n";
	print STATIST "<parameter>\n";
	print STATIST "<nodes value='$self->{nodes}' />\n";
	print STATIST "<maxspeed values='$self->{maxspeed}' />";
	print STATIST '</parameter>'."\n";
	
#=for comment
	print STATIST '<tracelevel id="AGT">'."\n";
	print STATIST ' <protocol id="udp" app="cbr">'."\n";
	print STATIST '  <agent id="cbr">'."\n   ";

	while ( my ($key,$val) = each %{$self->{AGT_CBR}} ) { print STATIST "<$key value=\"$val\"/>"; }
	print STATIST '  </agent>'."\n";
	print STATIST '  <agent id="tcp">'."\n";
	while ( my ($key,$val) = each %{$self->{AGT_TCP}} ) { print STATIST "<$key value=\"$val\"/>"; }
	print STATIST "\n  </agent>\n";
	print STATIST ' </protocol>'."\n";
	print STATIST '</tracelevel>'."\n";
	
  # RTR level 
	print STATIST '<tracelevel id="RTR">'."\n";
	print STATIST ' <protocol id="AODV">';
	print STATIST '  <type id="AODV">';
	while ( my ($key,$val) = each %{$self->{RTR_AODV}}){ print STATIST "<$key value=\"$val\"/>"; }
	print STATIST "\n  </type>\n";
	
	print STATIST '  <type id="AODVRREQ">';
	while ( my ($key,$val) = each %{$self->{RTR_AODVRREQ}} )  { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n   </type>\n";
	
	print STATIST '  <type id="AODVRREP">'."\n";
	while ( my ($key,$val) = each %{$self->{RTR_AODVRREP}} )  { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n  </type>\n";
	
	print STATIST '  <type id="AODVERROR">';
	while ( my ($key,$val) = each %{$self->{RTR_AODVERROR}} ) { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n  </type>\n";
	
	print STATIST '  <type id="AODVHELLO">';
	while ( my ($key,$val) = each %{$self->{RTR_AODVHELLO}} ) { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n  </type>\n";
	
	print STATIST "  <type id=\"AODVUNKOWN\" value='$self->{RTR_AODVUNKNOWN}'/>";	
	print STATIST " </protocol>\n";
	
	print STATIST ' <protocol id="CBR">'."\n ";
	while ( my ($key,$val) = each %{$self->{RTR_CBR}} ) { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n </protocol>\n";
	print STATIST ' <protocol id="TCP">'."\n ";
	while ( my ($key,$val) = each %{$self->{RTR_TCP}} ) { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n </protocol>\n";
	print STATIST '</tracelevel>';
	
	#MAC level
	print STATIST '<tracelevel id="MAC">'."\n";
	
	print STATIST ' <protocol id="AODV">'."\n";
	while ( my ($key,$val) = each %{$self->{MAC_AODV}} )  { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n </protocol>\n";
	print STATIST ' <protocol id="cbr">'."\n";
	while ( my ($key,$val) = each %{$self->{MAC_CBR}} )   { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n </protocol>";
	print STATIST ' <protocol id="tcp">'."\n";
	while ( my ($key,$val) = each %{$self->{MAC_TCP}} )   { print STATIST "<$key value='$val'/>"; }
	print STATIST "\n </protocol>";

	print STATIST "\n</tracelevel>\n";

#=cut

	print STATIST "</stat>";

} 

=head2 saveAsText

	Save statistics as pure text.

=cut

sub saveAsText {
	my $self =shift;
	my $statfile=shift; 
#=for comment
	
	#if ($statfile eq ""){$statfile="statistic.txt";}
	if ($statfile eq "") { die("No Parameter given where the statfile should be. \n"); }
	
	print(STDERR "save Statistics at $statfile .\n");
	open (STATIST,">$statfile") or die("\n Couldn't open $statfile .\n");

	print STATIST "# Nodes:$self->{nodes} MaxSpeed:$self->{maxspeed}\n";
	print STATIST "# send recv forward drop unknown\n";
	print STATIST "### AGT ###\n";
	print STATIST "CBR ".$self->{AGT_CBR}{s}." ".$self->{AGT_CBR}{r}." ".$self->{AGT_CBR}{f}." ".$self->{AGT_CBR}{d}." ".$self->{AGT_CBR}{u}."\n";
	print STATIST "TCP ".$self->{AGT_TCP}{s}." ".$self->{AGT_TCP}{r}." ".$self->{AGT_TCP}{f}." ".$self->{AGT_TCP}{d}." ".$self->{AGT_TCP}{u}."\n";	
	print STATIST "UNKNOWN ".$self->{AGT_UNKNOWN}{s}." ".$self->{AGT_UNKNOWN}{r}." ".$self->{AGT_UNKNOWN}{f}." ".$self->{AGT_UNKNOWN}{d}." ".$self->{AGT_UNKNOWN}{u}."\n";	
	print STATIST "\n";	
	
	print STATIST "### AGT Bytes ###\n";
	print STATIST "CBR ".$self->{AGT_CBR}{sB}." ".$self->{AGT_CBR}{rB}." ".$self->{AGT_CBR}{fB}." ".$self->{AGT_CBR}{dB}." ".$self->{AGT_CBR}{uB}."\n";
	print STATIST "TCP ".$self->{AGT_TCP}{sB}." ".$self->{AGT_TCP}{rB}." ".$self->{AGT_TCP}{fB}." ".$self->{AGT_TCP}{dB}." ".$self->{AGT_TCP}{uB}."\n";
	print STATIST "\n";
	
	print STATIST "### RTR ###\n";
	print STATIST "CBR        ".$self->{RTR_CBR}{s}." ".$self->{RTR_CBR}{r}." ".$self->{RTR_CBR}{f}." ".$self->{RTR_CBR}{d}." ".$self->{RTR_CBR}{u}."\n";
	print STATIST "TCP        ".$self->{RTR_TCP}{s}." ".$self->{RTR_TCP}{r}." ".$self->{RTR_TCP}{f}." ".$self->{RTR_TCP}{d}." ".$self->{RTR_TCP}{u}."\n";
	print STATIST "AODV-RREQ  ".$self->{RTR_AODVRREQ}{s}." ".$self->{RTR_AODVRREQ}{r}." ".$self->{RTR_AODVRREQ}{f}." ".$self->{RTR_AODVRREQ}{d}." ".$self->{RTR_AODVRREP}{u}."\n";
	print STATIST "AODV-RREP  ".$self->{RTR_AODVRREP}{s}." ".$self->{RTR_AODVRREP}{r}." ".$self->{RTR_AODVRREP}{f}." ".$self->{RTR_AODVRREP}{d}." ".$self->{RTR_AODVRREP}{u}."\n";
	print STATIST "AODV-HELLO ".$self->{RTR_AODVHELLO}{s}." ".$self->{RTR_AODVHELLO}{r}." ".$self->{RTR_AODVHELLO}{f}." ".$self->{RTR_AODVHELLO}{d}." ".$self->{RTR_AODVHELLO}{u}."\n";
	print STATIST "AODV-ERROR ".$self->{RTR_AODVERROR}{s}." ".$self->{RTR_AODVERROR}{r}." ".$self->{RTR_AODVERROR}{f}." ".$self->{RTR_AODVERROR}{d}." ".$self->{RTR_AODVERROR}{u}."\n";
	print STATIST "AODV       ".$self->{RTR_AODV}{s}." ".$self->{RTR_AODV}{r}." ".$self->{RTR_AODV}{f}." ".$self->{RTR_AODV}{d}." ".$self->{RTR_AODV}{u}."\n";
	print STATIST "UNKNOWN ".$self->{RTR_UNKNOWN}{s}." ".$self->{RTR_UNKNOWN}{r}." ".$self->{RTR_UNKNOWN}{f}." ".$self->{RTR_UNKNOWN}{d}." ".$self->{RTR_UNKNOWN}{u}."\n";	
	
	print STATIST "\n";
	print STATIST "### RTR Bytes ###\n";
	print STATIST "CBR        ".$self->{RTR_CBR}{sB}." ".$self->{RTR_CBR}{rB}." ".$self->{RTR_CBR}{fB}." ".$self->{RTR_CBR}{dB}." ".$self->{RTR_CBR}{uB}."\n";
	print STATIST "TCP        ".$self->{RTR_TCP}{sB}." ".$self->{RTR_TCP}{rB}." ".$self->{RTR_TCP}{fB}." ".$self->{RTR_TCP}{dB}." ".$self->{RTR_TCP}{uB}."\n";
	print STATIST "AODV-RREQ  ".$self->{RTR_AODVRREQ}{sB}." ".$self->{RTR_AODVRREQ}{rB}." ".$self->{RTR_AODVRREQ}{fB}." ".$self->{RTR_AODVRREQ}{dB}." ".$self->{RTR_AODVRREQ}{uB}."\n";
	print STATIST "AODV-RREP  ".$self->{RTR_AODVRREP}{sB}." ".$self->{RTR_AODVRREP}{rB}." ".$self->{RTR_AODVRREP}{fB}." ".$self->{RTR_AODVRREP}{dB}." ".$self->{RTR_AODVRREP}{uB}."\n";
	print STATIST "AODV-HELLO ".$self->{RTR_AODVHELLO}{sB}." ".$self->{RTR_AODVHELLO}{rB}." ".$self->{RTR_AODVHELLO}{fB}." ".$self->{RTR_AODVHELLO}{dB}." ".$self->{RTR_AODVHELLO}{uB}."\n";
	print STATIST "AODV-ERROR ".$self->{RTR_AODVERROR}{sB}." ".$self->{RTR_AODVERROR}{rB}." ".$self->{RTR_AODVERROR}{fB}." ".$self->{RTR_AODVERROR}{dB}." ".$self->{RTR_AODVERROR}{uB}."\n";
	print STATIST "AODV       ".$self->{RTR_AODV}{sB}." ".$self->{RTR_AODV}{rB}." ".$self->{RTR_AODV}{fB}." ".$self->{RTR_AODV}{dB}." ".$self->{RTR_AODV}{uB}."\n";
  
	print STATIST "\n";	
	print STATIST "### MAC ###\n";
	print STATIST "CBR        ".$self->{MAC_CBR}{s}." ".$self->{MAC_CBR}{r}." ".$self->{MAC_CBR}{f}." ".$self->{MAC_CBR}{d}." ".$self->{MAC_CBR}{u}."\n";
	print STATIST "TCP        ".$self->{MAC_TCP}{s}." ".$self->{MAC_TCP}{r}." ".$self->{MAC_TCP}{f}." ".$self->{MAC_TCP}{d}." ".$self->{MAC_TCP}{u}."\n";
	print STATIST "AODV-RREQ  ".$self->{MAC_AODVRREQ}{s}." ".$self->{MAC_AODVRREQ}{r}." ".$self->{MAC_AODVRREQ}{f}." ".$self->{MAC_AODVRREQ}{d}." ".$self->{MAC_AODVRREQ}{u}."\n";
	print STATIST "AODV-RREP  ".$self->{MAC_AODVRREP}{s}." ".$self->{MAC_AODVRREP}{r}." ".$self->{MAC_AODVRREP}{f}." ".$self->{MAC_AODVRREP}{d}." ".$self->{MAC_AODVRREP}{u}."\n";
	print STATIST "AODV-HELLO ".$self->{MAC_AODVHELLO}{s}." ".$self->{MAC_AODVHELLO}{r}." ".$self->{MAC_AODVHELLO}{f}." ".$self->{MAC_AODVHELLO}{d}." ".$self->{MAC_AODVHELLO}{u}."\n";
	print STATIST "AODV-ERROR ".$self->{MAC_AODVERROR}{s}." ".$self->{MAC_AODVERROR}{r}." ".$self->{MAC_AODVERROR}{f}." ".$self->{MAC_AODVERROR}{d}." ".$self->{MAC_AODVERROR}{u}."\n";
	print STATIST "AODV       ".$self->{MAC_AODV}{s}." ".$self->{MAC_AODV}{r}." ".$self->{MAC_AODV}{f}." ".$self->{MAC_AODV}{d}." ".$self->{MAC_AODV}{u}."\n";	
	print STATIST "UNKNOWN ".$self->{MAC_UNKNOWN}{s}." ".$self->{MAC_UNKNOWN}{r}." ".$self->{MAC_UNKNOWN}{f}." ".$self->{MAC_UNKNOWN}{d}." ".$self->{MAC_UNKNOWN}{u}."\n";

	print STATIST "\n";
	print STATIST "### MAC Bytes ###\n";	
	print STATIST "CBR        ".$self->{MAC_CBR}{sB}." ".$self->{MAC_CBR}{rB}." ".$self->{MAC_CBR}{fB}." ".$self->{MAC_CBR}{dB}." ".$self->{MAC_CBR}{uB}."\n";
	print STATIST "TCP        ".$self->{MAC_TCP}{sB}." ".$self->{MAC_TCP}{rB}." ".$self->{MAC_TCP}{fB}." ".$self->{MAC_TCP}{dB}." ".$self->{MAC_TCP}{uB}."\n";
	print STATIST "AODV-RREQ  ".$self->{MAC_AODVRREQ}{sB}." ".$self->{MAC_AODVRREQ}{rB}." ".$self->{MAC_AODVRREQ}{fB}." ".$self->{MAC_AODVRREQ}{dB}." ".$self->{MAC_AODVRREP}{uB}."\n";
	print STATIST "AODV-RREP  ".$self->{MAC_AODVRREP}{sB}." ".$self->{MAC_AODVRREP}{rB}." ".$self->{MAC_AODVRREP}{fB}." ".$self->{MAC_AODVRREP}{dB}." ".$self->{MAC_AODVRREP}{uB}."\n";
	print STATIST "AODV-HELLO ".$self->{MAC_AODVHELLO}{sB}." ".$self->{MAC_AODVHELLO}{rB}." ".$self->{MAC_AODVHELLO}{fB}." ".$self->{MAC_AODVHELLO}{dB}." ".$self->{MAC_AODVHELLO}{uB}."\n";
	print STATIST "AODV-ERROR ".$self->{MAC_AODVERROR}{sB}." ".$self->{MAC_AODVERROR}{rB}." ".$self->{MAC_AODVERROR}{fB}." ".$self->{MAC_AODVERROR}{dB}." ".$self->{MAC_AODVERROR}{uB}."\n";
	print STATIST "AODV       ".$self->{MAC_AODV}{sB}." ".$self->{MAC_AODV}{rB}." ".$self->{MAC_AODV}{fB}." ".$self->{MAC_AODV}{dB}." ".$self->{MAC_AODV}{uB}."\n";
	print STATIST "sum end2end delays: $self->{sumAGT_end_to_end_delay}\n";
	print STATIST "     end2end-delay: $self->{AGT_end_to_end_delay}\n";	
	
	print STATIST "\n";
	print STATIST "### IFQ (drops) ###\n";
	print STATIST "IFQ  ".$self->{IFQ}{s}." ".$self->{IFQ}{r}." ".$self->{IFQ}{f}." ".$self->{IFQ}{d}." ".$self->{IFQ}{u}."\n\n";	
	print STATIST "### UNKNOWN LEVEL ###\n";
	print STATIST "UNKNOWN  ".$self->{UNKNOWN}{s}." ".$self->{UNKNOWN}{r}." ".$self->{UNKNOWN}{f}." ".$self->{UNKNOWN}{d}." ".$self->{UNKNOWN}{u}."\n";
	close STATIST or die("Couldn't close $statfile.\n");
	
#=cut
}

sub interfacequeuetrace{
}

sub agenttrace{
}

sub mactrace{
}

sub routertrace{
}

sub getTracelevel {
}


sub getBaseInfos{
	my @values=split(/\s+/,$_);
	my $time;
	my $id=0;
	my $col=0;
	foreach (@values) {
		if ($_ eq  "-t")  { $time = $values[$col+1];}
		if ($_ eq  "-Ii")  { $id = $values[$col+1];}
		$col++;
	}
	#print STDERR "$time $id\n";
	return ($time,$id);
}

sub getPacketSize {
	my $size=0;
	my $col=0;
	#split the row
	my @values=split(/\s+/,$_);
	foreach (@values) {
		#search packet length 
		if ($_ eq  "-Il") { $size = $values[$col+1]}
		$col++;
	}
	#print(STDERR $size."\n");
	return  $size;
}
1;
__END__
