################################################
# Author: Peter Liscovius                      #    
# (c)2002                                      #
# software is under GPL licence                # 
#                                              #
# functions for analyzing ns-2 new-trace files #
# currently focussed on AODV                   #
################################################

# usage in other scripts: 
# use TraceStat;
# TraceStat::makeStat($filename [$maxspeed] );

# To get a value: 
# print $TraceStat::AGT_CBR{s};

# To save that stat:
# TraceStat::saveStat([$statfile]);

# new objectoriented interface:
# use TraceStat;
# my $stat=new TraceStat;
# $stat->makeStat("datei");
# look into makestat.pl for an example.
# $statfile is optional and not important for stat calculation. 
# Default $statfile is "statistic.txt".
# $maxspeed is optional too. Just to have that value in the statfile.

# TODO: Add code so that TraceStat work real objectoriented stile, 
# so you can create different instances of TraceStat to compare
# results of different simulations (e.g. in TraceView or to create Graphs)
# the interface should the be something like:
# use TraceStat;
# $stat1=new TraceStat($filename);

# Version:0.3.5

use strict;

package TraceStatsimple;

sub new {
	my $class = shift;
	my $self= {};

	$self->{AGT_CBR} = {s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
	$self->{AGT_TCP} = {s=>0, r=>0, f=>0, d=>0, u=>0, sB=>0, rB=>0, fB=>0, dB =>0, uB=>0};
	
	$self->{nodes}=0;

	$self->{packet_id}=0;
	$self->{highest_packet_id}=0;

	# XXX fix me how to bless all these variables,hashs and arrays??
	bless ($self,	$class);
	return $self;
}

# interface: $stat->makeStat(tracefile,statisticfile); 
sub makeStat {
	my $self =shift;
	
	#$maxspeed is optional
	my ($file,$statfile,$maxspeed)=@_;
	print "Opening $file and generating statistic ..\n";
	
	open (FILE, "<$file") || die("Couldn't open file $file in TraceStat::makeStat()..\n");
	# Problem when big files and small RAM in your workstation. 
	# Then better read the file linebyline while doing stat but script then slower.
	my @file = <FILE>; 
	close FILE;

	#my $maxspeed;
	my @firstline=split(/ /,$file[0]);
	$self->{nodes}=$firstline[3];
	my $time;
foreach (@file)	{
	#detect the tracelevel
	if ($_ =~ /^[srfd]\s/){
		($time,$self->{packet_id})=getBaseInfos($_);
	
	} else {
		print(STDERR "Irregular line in tracefile:\n $_");
		next;
	}
	#print STDERR "$time $self->{$packed_id}\n";
	if ($self->{packet_id} > $self->{highest_packet_id}) {$self->{highest_packet_id} = $self->{packet_id};}
	
	if ($_ =~ /-Nl AGT/){ 
		# We are tracing in the Agent-layer
		# Counting a send packet here doesn't automatically means it is really sent over the air. 
		# Could be also dropped in interfacequeue e.g. 
		if ($_ =~ /-It cbr /i){	
			#print STDERR "Constant Bit Rate (UDP?)-packets";
			if ($_ =~ /^s/){	
				$self->{AGT_CBR}{sB}+=getPacketSize($_);
				$self->{AGT_CBR}{s}++;
				if ( not defined $self->{AGT_start_time}[$self->{packet_id}]) {$self->{AGT_start_time}[$self->{packet_id}] = $time;}
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{AGT_CBR}{rB}+=getPacketSize($_);
				$self->{AGT_CBR}{r}++;
				if (not defined $self->{AGT_end_time}[$self->{packet_id}]) {$self->{AGT_end_time}[$self->{packet_id}] = $time; }
			} 
			elsif ($_ =~ /^f/){  
				$self->{AGT_CBR}{fB}+=getPacketSize($_);
				$self->{AGT_CBR}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				$self->{AGT_CBR}{dB}+=getPacketSize($_);
				$self->{AGT_CBR}{d}++;
			}
			else {	
				$self->{AGT_CBR}{uB}+=getPacketSize($_);
				$self->{AGT_CBR}{u}++;
			}	
		}
		if ($_ =~ /-It tcp /i){
			#print STDERR "Constant Bit Rate (UDP?)-packets";
			if ($_ =~ /^s/){	
				$self->{AGT_TCP}{sB}+=getPacketSize($_);
				$self->{AGT_TCP}{s}++;
				if ( not defined $self->{AGT_start_time}[$self->{packet_id}]) {$self->{AGT_start_time}[$self->{packet_id}] = $time;}
			}
			elsif ($_ =~ /^r/){	
				#print STDERR " was received\n";
				$self->{AGT_TCP}{rB}+=getPacketSize($_);
				$self->{AGT_TCP}{r}++;
				if (not defined $self->{AGT_end_time}[$self->{packet_id}]) {$self->{AGT_end_time}[$self->{packet_id}] = $time; }
			} 
			elsif ($_ =~ /^f/){  
				$self->{AGT_TCP}{fB}+=getPacketSize($_);
				$self->{AGT_TCP}{f}++;
			}
			elsif ($_ =~ /^d/){ 	
				$self->{AGT_TCP}{dB}+=getPacketSize($_);
				$self->{AGT_TCP}{d}++;
			}
			else {	
				$self->{AGT_TCP}{uB}+=getPacketSize($_);
				$self->{AGT_TCP}{u}++;
			}
		} 

	}

}

	
print(STDERR "calculating delay ..\n");
my $sumAGT_end_to_end_delay=0;
for (my $packet_id = 0; $packet_id <= $self->{highest_packet_id} ; $packet_id++) {
	
	if ((defined $self->{AGT_end_time}[$self->{packet_id}]) && (defined $self->{AGT_start_time}[$self->{packet_id}]) ) {
		my $packet_duration = $self->{AGT_end_time}[$self->{packet_id}] - $self->{AGT_start_time}[$self->{packet_id}];
		print(STDERR "pkt: $packet_id delay: $packet_duration\n");
		if ($packet_duration >0) {$sumAGT_end_to_end_delay += $packet_duration;}
	}
}
#if ( ($self->{AGT_CBR}->{r}+$self->{AGT_TCP}->{r}) < 1) { $self->{end_to_end_delay}=0;}
#else { $self->{AGT_end_to_end_delay} = $sumAGT_end_to_end_delay / ($self->{AGT_CBR}->{r} + $self->{AGT_TCP}->{r}); }

}

=head2 saveAsText

A method to save the Stats as plain text.

=cut
sub saveAsText {
	my $self=shift;
	my $statfile=shift;
#=for Auskommentieren
	
	#if ($statfile eq ""){$statfile="statistic.txt";}
	if ($statfile eq "") { die("No Parameter given where the statfile should be. \n"); }
	
	print(STDERR "save Statistics at $statfile .\n");
	open (STATIST,">$statfile") or die("\n Couldn't open $statfile .\n");

	print STATIST "# Nodes:".$self->{nodes}." MaxSpeed:".$self->{maxspeed}."\n";
	print STATIST "# send recv forward drop unknown\n";
	print STATIST "### AGT ###\n";
	print STATIST "CBR ".$self->{AGT_CBR}{s}." ".$self->{AGT_CBR}{r}." ".$self->{AGT_CBR}{f}." ".$self->{AGT_CBR}{d}." ".$self->{AGT_CBR}{u}."\n";
	
#=cut
}


=head2 saveAsXML

A method to save the Stats as XML. Very rudimentary. More an idea than useful. 

Enables us to calculate the stats before using traceview without doublecalculating stats. Traceview can load
then the xmldata by perl(serverside) and build html/swf/whatever or even let flash(clientside) load 
the xmldata a parse there.

=cut

sub saveAsXML {
	my $self=shift;
	my $statfile=shift; 

#=for Auskommentieren
	
	#if ($statfile eq ""){$statfile="statistic.txt";}
	if ($statfile eq "") { die("No Parameter given where the statfile should be. \n"); }
	
	print(STDERR "save Statistics at $statfile .\n");
	open (STATIST,">$statfile") or die("\n Couldn't open $statfile .\n");

	print STATIST "<\?xml version='1.0'\?><nsstat>\n";
	print STATIST "<parameter>\n";
	print STATIST "<nodes value='$self->{nodes}' />\n";
	print STATIST "<maxspeed values='$self->{maxspeed}' />\n";
	print STATIST "</parameter>\n";
	
	print STATIST "<AGENT>\n";
	
	print STATIST "<CBR>\n";
	print STATIST "<send packets='$self->{AGT_CBR}{s}' bytes='$self->{AGT_CBR}{sB}'/>\n";
	print STATIST "<forward packets='$self->{AGT_CBR}{f}' bytes='$self->{AGT_CBR}{fB}' />\n";	
	print STATIST "<receive packets='$self->{AGT_CBR}{r}' bytes='$self->{AGT_CBR}{rB}' />\n";
	print STATIST "<drop packets='$self->{AGT_CBR}{f}' bytes='$self->{AGT_CBR}{fB}' />\n";
	print STATIST "<unknown packets='$self->{AGT_CBR}{u}' bytes='$self->{AGT_CBR}{uB}' />\n";
	print STATIST "</CBR>\n";
	
	print STATIST "<TCP>\n";
	print STATIST "<send packets='$self->{AGT_TCP}{s}' bytes='$self->{AGT_TCP}{sB}' />\n";
	print STATIST "<forward packets='$self->{AGT_TCP}{f}' bytes='$self->{AGT_TCP}{fB}' />\n";	
	print STATIST "<receive packets='$self->{AGT_TCP}{r}' bytes='$self->{AGT_TCP}{rB}' />\n";
	print STATIST "<drop packets='$self->{AGT_TCP}{f}' bytes='$self->{AGT_TCP}{fB}' />\n";
	print STATIST "<unknown packets='$self->{AGT_TCP}{u}' bytes='.$self->{AGT_TCP}{uB}' />\n";
	print STATIST "</TCP>\n";
	
	print STATIST "</AGENT>\n";
	
	print STATIST "</nsstat>";

#=cut

}
sub interfacequeuetrace{
}

sub agenttrace{
}

sub mactrace{
}

sub routertrace{
}

sub getTracelevel {
}


sub getBaseInfos{
	my @values=split(/\s+/,$_);
	my $time;
	my $id=0;
	my $col=0;
	foreach (@values) {
		if ($_ eq  "-t")  { $time = $values[$col+1];}
		if ($_ eq  "-Ii")  { $id = $values[$col+1];}
		$col++;
	}
	#print STDERR "$time $id\n";
	return ($time,$id);
}

sub getPacketSize {
	my $size=0;
	my $col=0;
	#split the row
	my @values=split(/\s+/,$_);
	foreach (@values) {
		#search packet length 
		if ($_ eq  "-Il") { $size = $values[$col+1]}
		$col++;
	}
	#print(STDERR $size."\n");
	return  $size;
}
1;
__END__
