#!/usr/bin/perl -w

use strict;

my $basedir=`pwd`;
chomp($basedir);
my $dir=$basedir."/out";

my @nodes = qw(10); # a list of nodes e.g: qw(5 10 20 50) 

# look into movementgenerator.pl for more
my @maxspeed = qw(5); # meter/sec  (1 2 5 10 20 50)
my $movements=1; # number of movement files with same setdest-parameters ~ Wieviel Durchlufe?
my $movementseed;
my $x=2000; # in meter
my $y=500;  # in meter

my $simtime=50.0; # in seconds 
# Simulationsdurchlaeufe =$#nodes * $#maxspeed * $movements;

# datatrafficgenerator
my $trafficGenerator="connectiontrafficgenerator.tcl";
my $connectionTrafficDir=`pwd`;
chop($connectionTrafficDir);
print "connectionTrafficDir=$connectionTrafficDir\n";
my $connectionTrafficFile;
my $ctrate="1"; # in packets per second, packetsize is in tcl-script
my $cttype="cbr"; # cbr, not tested: ftp

# movement directory 
my $movementDir="movement";
my $movementFile;



my $tr=$dir."/tracefile.tr";
# I don't do namtrace in the tclscript, but let it for people who want.
my $namtr=$dir."/namtracefile.nam";

&cleanup;
&doit;
# post-big-simulation-scripting



sub cleanup {
  chdir("$dir") or die("Couldn't change to $dir in cleanup routine. I better end ..");
  opendir(DIR,".");
  my @file = readdir(DIR);
  closedir(DIR);
  foreach (@file) {
	# only delete tracefiles, namtracefiles, samples and queuefiles and htmlfiles.
 	if (-f $_ && (/tr$/ or /nam$/ or /^sample/ or /^queue/ or /html$/ or /htm$/ or /xml$/) ) { 
		print "cleanup:$_\n";
		unlink($_) or print STDERR ("couldn't delete $_.\n");
	}
  }
  chdir("$basedir");
}

sub doit {
	foreach my $nn (@nodes)
	{
		# generate connection traffic (CBR)
		# connectiontraffic has a seed option
		print STDERR "generating connectiontraffic for $nn nodes ..\n";
		$connectionTrafficFile="$connectionTrafficDir/traffic-$nn.txt";
		my $ctgeneration="ns $trafficGenerator -type $cttype -nn $nn -seed 1 -mc $nn -rate $ctrate >$connectionTrafficFile";
		print "$ctgeneration\n\n";
		my $status=system("$ctgeneration");
		if ($status !=0) { die("generation of connectiontraffic failed .. exit simulation perlscript.\n");}


		foreach my $maxspeed (@maxspeed) {
		  for ($movementseed=1; $movementseed<=$movements; $movementseed++) {
				# I use pregenerated files by movementgenerator.pl.
				$movementFile = $movementDir."/movement-n$nn-x$x-y$y-t$simtime-s$maxspeed-".$movementseed.".txt";
				print "movementFile=".$movementFile."\n";
		
				if (not -r $movementFile) { die("Couldn't read $movementFile . Create this movementfile, look if file is readable and restart sim.\n"); }
		
				#make simulation
				my $sim="ns aodvsim.tcl -nn $nn -x $x -y $y -sc $movementFile -cp $connectionTrafficFile -stop $simtime -tr $tr -namtr $namtr >$dir/log_aodvtables.html 2>$dir/log_aodverror.txt";
				print "$sim\n";
				$status = system("$sim"); 
				if ($status !=0) { die("ns-simulation aborted :-(\n Look into $dir/log_aodvtables.html and $dir/log_aodverror.txt to find reasons why ns aborted.\n Now I kill me ..\n"); }
    
				# post-every-simulation-scripting 
				&moveSim($nn,$maxspeed,$movementseed);
			}
		}
	}
}

# moves the current simresults in 'out/' in a deeper directory.
sub moveSim {
	print "move sim from out/ to an extra subdirectory\n";
	my ($nodes,$speed,$mseed)=@_;

	chdir("$dir");
	opendir(DIR,".");
	my @filelist = readdir(DIR);
	closedir(DIR);

	my $saveDir = "nodes".$nodes."-".$x."x".$y."-speed".$speed."-mseed".$mseed;
	mkdir($saveDir,0777);	
	foreach my $file (@filelist) {
 		#if (-f $file && $file =~ /^s/) {
		if (-f $file) {
			print "saveSim: move $file to $saveDir\n";
			my $status=system("mv $file $saveDir");  
			if ($status != 0) {print STDERR "couldn't copy $file to $saveDir.\n"; }
		}
	}
	chdir("$basedir");	
	
	#save also the generated trafficfile
	my $status=system("cp $connectionTrafficFile $dir\/$saveDir");  
	if ($status != 0) {print STDERR "Couldn't copy $connectionTrafficFile to $dir\/$saveDir.\n"; }
	#save also the used movementfile
	$status=system("cp $movementFile $dir\/$saveDir");  
	if ($status != 0) {print STDERR "Couldn't copy $movementFile to $dir\/$saveDir.\n"; }
}
